/* s2uc10.c: demonstration / use case 10 for S2PLOT library.
 *
 * David Barnes, October 2006
 *
 * $Id: s2uc10.c,v 1.10 2007/04/04 22:58:45 dbarnes Exp $
 *
 * Demonstrates plotting points and attaching handles.  Press Shift-S
 * to toggle selection mode, and Shift-C to toggle crosshair.
 *
 */

#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include "s2plot.h"

#define XLIMIT 100.0
#define YLIMIT 100.0
#define ZLIMIT 10.0
#define RANDSEED 103491794l

void initcallback(void);
void pickcb(int *id);
void newcallback(double *time, int *keycount);

int main(int argc, char **argv) {

  float x1 = -XLIMIT, x2 = XLIMIT; /* x world coord range */
  float y1 = -YLIMIT, y2 = YLIMIT; /* y world coord range */
  float z1 = -ZLIMIT, z2 = ZLIMIT; /* z world coord range */

  char *xopt = "BCDET";           /* B = draw front bottom (-Y,-Z) edge 
				    * C = draw front top (-Y,+Z) edge
				    * D = draw back bottom (+Y,-Z) edge
				    * E = draw back top (+Y,+Z) edge
				    * T = draw major tick marks
				    * S = draw minor tick marks
				    */
  char *yopt = "BCDET";           /* B = draw bottom left (-X,-Z) edge
				    * C = draw bottom right (+X,-Z) edge
				    * D = draw top left (-X,+Z) edge
				    * E = draw top right (+X,+Z) edge
				    */
  char *zopt = "BCDET";           /* B = draw left front (-X,-Y) edge
				    * C = draw right front (+X,-Y) edge
				    * D = draw left back (-X,+Y) edge
				    * E = draw right back (+X,+Y) edge
				    */

  /* major tick increments on all axes */
  float xtick = 0.0, ytick = 0.0, ztick = 0.0;

  /* number of subdivisions on all axes */
  int nxsub = 4, nysub = 1, nzsub = 10;

  int interactive = 1; /* 0 = just draw; 1 = draw and go interactive */

  /* open S2PLOT device */
  fprintf(stderr, "Please note, selection is not enabled in all environments.\n");
  fprintf(stderr, "Press \"Shift-S\" to enable selection in this demo.\n");
  fprintf(stderr, "Press \"Shift-C\" to enable crosshairs.\n");
  if (!s2opend("/?", argc, argv)) {
    fprintf(stderr, "Failed to open S2PLOT device - exiting!\n");
    exit(-1);
  }

  /* set world coordinate range */
  s2swin(x1, x2, y1, y2, z1, z2);

  /* draw and label a box around the points, in white */
  s2sci(S2_PG_WHITE);
  s2box(xopt, xtick, nxsub, yopt, ytick, nysub, zopt, ztick, nzsub);

  initcallback();
  cs2shcb(&pickcb);
  cs2scb(&newcallback);

  /* flush graphics and enter interactive mode */
  s2show(interactive);

  /* successful run */
  return 0;

}

/* number of points */
#define NP 10
/* storage for point population */
float *x = 0, *y = 0, *z = 0;
/* mask of selected points */
int *selected;

void initcallback() {
  int i;

  /* fetch bounds of our world */
  float x1, x2, y1, y2, z1, z2;
  s2qwin(&x1, &x2, &y1, &y2, &z1, &z2);

  srand(RANDSEED);
  x = (float *)calloc(NP, sizeof(float));
  y = (float *)calloc(NP, sizeof(float));
  z = (float *)calloc(NP, sizeof(float));
  selected = (int *)calloc(NP, sizeof(int));
  for (i = 0; i < NP; i++) {
    x[i] = x1 + rand() / (float)RAND_MAX * (x2 - x1);
    y[i] = y1 + rand() / (float)RAND_MAX * (y2 - y1);
    z[i] = z1 + rand() / (float)RAND_MAX * (z2 - z1);
    selected[i] = 0;
  }
}

/* the callback function that is told when a handle is clicked on */
void pickcb(int *id) {
  /* we know that our id maps directly to our array location, so 
   * toggle directly */
  selected[*id] = (selected[*id] + 1) % 2;

  /* we also need to toggle the "S2PLOT internal state" of this handle
   * in case callbacks are not currently running */
  cs2th(*id);
  
  /* do nothing else for now */

}

/* the callback function which will construct dynamic geometry to be 
 * drawn.
 */
void newcallback(double *time, int *keycount) {
  //static double phase = 0.;
  int i;

  /* fetch bounds of our world */
  float x1, x2, y1, y2, z1, z2;
  s2qwin(&x1, &x2, &y1, &y2, &z1, &z2);

  /* jitter the points */
  for (i = 0; i < NP; i++) {
    x[i] += (rand() / (float)RAND_MAX - 0.5) * (x2 - x1) / 133.0;
    y[i] += (rand() / (float)RAND_MAX - 0.5) * (y2 - y1) / 133.0;
    z[i] += (rand() / (float)RAND_MAX - 0.5) * (z2 - z1) / 133.0;
  }

  /* draw the points */
  s2sci(S2_PG_CYAN);
  s2slw(2);
  s2pt(NP, x, y, z, 1);

  /* attach handles */
  COLOUR col = {0.99,0.05,0.05};
  COLOUR hilite = {0.99,0.99,0.05};
  XYZ pos;
  for (i = 0; i < NP; i++) {
    pos.x = x[i];
    pos.y = y[i];
    pos.z = z[i];
    cs2ah(pos, 0.33, col, hilite, i, selected[i]);
  }
}
