/* s2uc2.c: use case 2 for S2PLOT library.
 *
 * David Barnes, October 2005
 *
 * $Id: s2uc2.c,v 3.1 2006/12/22 03:33:25 dbarnes Exp $
 *
 * Demonstrates multiple plots via viewport settings, coordinate 
 * labelling, surfaces and flat plots, and parametric functions.
 *
 */

#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include "s2plot.h"

#define XLIMIT 10.0
#define YLIMIT 10.0
#define ZLIMIT 50.0
#define TWOPI 6.2831853

inline float fx(float *t) { 
  return XLIMIT * sin(*t); }
inline float fy(float *t) { 
  return YLIMIT * *t / TWOPI * cos(*t); }
inline float fz(float *t) { 
  return 20. + (ZLIMIT * *t / TWOPI + 4.*sin(5* *t)*sin(3* *t))*0.5; }

int main(int argc, char **argv) {

  /* coord system is as follows:
   * X is negative left on screen, positive right on screen
   * Y is negative front of screen, positive behind screen
   * Z is positive up on screen, negative down on screen
   */

  int fullscreen = 1; /* full-screen mode */
  int stereo = 2;     /* passive stereo mode (0 for mono, 1 for active st.) */

  float x1 = -XLIMIT, x2 = XLIMIT; /* x world coord range */
  float y1 = -YLIMIT, y2 = YLIMIT; /* y world coord range */
  float z1 = -ZLIMIT/4.0, z2 = ZLIMIT; /* z world coord range */

  char *xopt = "ABCDETSMNG";           /* B = draw front bottom (-Y,-Z) edge 
				    * C = draw front top (-Y,+Z) edge
				    * D = draw back bottom (+Y,-Z) edge
				    * E = draw back top (+Y,+Z) edge
				    * T = draw major tick marks
				    * S = draw minor tick marks
				    */
  char *yopt = "ABCDETSMNG";           /* B = draw bottom left (-X,-Z) edge
				    * C = draw bottom right (+X,-Z) edge
				    * D = draw top left (-X,+Z) edge
				    * E = draw top right (+X,+Z) edge
				    */
  char *zopt = "ABCDETSMNG";           /* B = draw left front (-X,-Y) edge
				    * C = draw right front (+X,-Y) edge
				    * D = draw left back (-X,+Y) edge
				    * E = draw right back (+X,+Y) edge
				    */

  /* major tick increments on all axes */
  float xtick = 0.0, ytick = 0.0, ztick = 0.0;

  /* number of subdivisions on all axes */
  int nxsub = 4, nysub = 1, nzsub = 10;

  int i; /* loop variable/s */

  int interactive = 1; /* 0 = just draw; 1 = draw and go interactive */

  /* open S2PLOT device */
  if (!s2open(fullscreen, stereo, argc, argv)) {
    fprintf(stderr, "Failed to open S2PLOT device - exiting!\n");
    exit(-1);
  }

  /* set viewport */
  /* since we only plot one graph here, the only change should be that 
   * we get a stretched box instead of a cube */
  s2svp(-1,0,-1,0,-2,0);

  /* set world coordinate range */
  s2swin(x1, x2, y1, y2, z1, z2);

  /* Let's start by generating a data set to work with. */
  float **image;
  #define NXI 64
  #define NYI 64
  float minz = 9e30;
  float maxz = -9e30;
  float r;
  int j;
  image = (float **)calloc(NXI, sizeof(float **));
  for (i = 0; i < NXI; i++) {
    image[i] = (float *)calloc(NYI, sizeof(float *));
    for (j = 0; j < NYI; j++) {
      r = (i-NXI/2)*(i-NXI/2) / (float)(NXI*NXI) + 
	(j-NYI/2)*(j-NYI/2) / (float)(NYI * NYI);
      r = 9.5 * sqrt(r);
      if (r > 0) {
	image[i][j] = 2.5 * sin(r)/r;
      } else {
	image[i][j] = 2.5;
      }
      r = (i-NXI/4) * (i-NXI/4) / (float)(NXI * NXI * 0.04) + 
	(j-NYI/3) * (j-NYI/3) / (float)(NYI * NYI * 0.04);
      r = 0.5 * r;
      if (r > 0) {
	image[i][j] -= 1.5 * sin(r)/r;
      } else {
	image[i][j] -= 1.5;
      }
      if (image[i][j] < minz) {
	minz = image[i][j];
      }
      if (image[i][j] > maxz) {
	maxz = image[i][j];
      }
    }
  }

  /* set up a unity transformation matrix */
  float tr[8];
  tr[0] = x1; tr[1] = (x2-x1)/(float)(NXI-1); tr[2] = 0.0;
  tr[3] = y1; tr[4] = 0.0; tr[5] = (y2-y1)/(float)(NYI-1);
  tr[6] = z1 - minz * (z2-z1)/(maxz-minz); tr[7] = (z2-z1)/(maxz-minz);

  /* install a cool colormap */
  s2icm("astro", 1000, 1400);
  s2scir(1000, 1400);

  /* plot surface */
  s2surp(image, NXI, NYI, 0, NXI-1, 0, NYI-1, minz, maxz, tr);

  /* plot the same data, but on the base-level z plane, using the 
   * more general s2surpa function.
   */
  float tr2[12];
  /* x transform */
  tr2[0] = tr[0];
  tr2[1] = tr[1];
  tr2[2] = tr[2];
  tr2[3] = 0.0;
  /* y transform */
  tr2[4] = tr[3];
  tr2[5] = tr[4];
  tr2[6] = tr[5];
  tr2[7] = 0.0;
  /* z "transform" */
  tr2[8] = 0.0;
  tr2[9] = 0.0; // tr[1]; // 0.0;
  tr2[10] = 0.0; // tr[5]; //0.0;
  tr2[11] = 0.0;
  /* new colormap and plot */
  s2icm("rainbow", 2000, 3000);
  s2scir(2000, 3000);
  s2surpa(image, NXI, NYI, 0, NXI-1, 0, NYI-1, minz, maxz, tr2);

  /* draw and label a box around the points, in a green-ish colour */
  s2sci(S2_PG_GREEN);
  s2sch(1.2);
  s2slw(1);
  s2box(xopt, xtick, nxsub, yopt, ytick, nysub, zopt, ztick, nzsub);

  /* change viewport - ie. draw plot somewhere else */
  s2svp(0.5,1.5,0.5,1.5,0.5,1.5);
  /* plot a  parametric line */
  s2slw(3.0);
  s2sci(S2_PG_YELLOW);
  s2funt(fx, fy, fz, 1000, -TWOPI, TWOPI);
  s2slw(1.0);
  s2sci(S2_PG_BLUE);
  s2box(xopt, xtick, nxsub, yopt, ytick, nysub, zopt, ztick, nzsub);

  /* flush graphics and enter interactive mode */
  s2show(interactive);

  /* successful run */
  return 0;

}
