/* s2uc3.c: demonstration use case 3 for S2PLOT library.
 *
 * David Barnes, October 2005
 *
 * $Id: s2uc3.c,v 3.4 2007/04/04 23:07:00 dbarnes Exp $
 *
 * Demonstrates drawing coordinate box, plotting coloured parametric
 * functions, various point symbols, and disks, circles and arrows,
 * drawn in different colours and line thicknesses.
 *
 */

#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include "s2plot.h"

#define XLIMIT 10.0
#define YLIMIT 20.0
#define ZLIMIT 50.0
#define TWOPI 6.2831853

inline float fx(float *t) { 
  return XLIMIT * fabs(*t / TWOPI) * sin(*t * 4); }
inline float fy(float *t) { 
  return YLIMIT * fabs(*t / TWOPI) * cos(*t * 4); }
inline float fz(float *t) { 
  return ZLIMIT * *t / TWOPI; }
inline float fc(float *t) { 
  return *t / TWOPI / 2.0 + 0.5; }

int main(int argc, char **argv) {

  int fullscreen = 0; /* not full-screen mode */
  int stereo = 0;     /* mono mode (1 for active stereo, 2 for passive) */

  float x1 = -XLIMIT, x2 = XLIMIT; /* x world coord range */
  float y1 = -YLIMIT, y2 = YLIMIT; /* y world coord range */
  float z1 = -ZLIMIT, z2 = ZLIMIT; /* z world coord range */

  char *xopt = "BCDETS";           /* B = draw front bottom (-Y,-Z) edge 
				    * C = draw front top (-Y,+Z) edge
				    * D = draw back bottom (+Y,-Z) edge
				    * E = draw back top (+Y,+Z) edge
				    * T = draw major tick marks
				    * S = draw minor tick marks
				    */
  char *yopt = "BCDETS";           /* B = draw bottom left (-X,-Z) edge
				    * C = draw bottom right (+X,-Z) edge
				    * D = draw top left (-X,+Z) edge
				    * E = draw top right (+X,+Z) edge
				    */
  char *zopt = "BCDETS";           /* B = draw left front (-X,-Y) edge
				    * C = draw right front (+X,-Y) edge
				    * D = draw left back (-X,+Y) edge
				    * E = draw right back (+X,+Y) edge
				    */

  /* major tick increments on all axes */
  float xtick = 0.0, ytick = 0.0, ztick = 0.0;

  /* number of subdivisions on all axes */
  int nxsub = 4, nysub = 1, nzsub = 10;

  int interactive = 1; /* 0 = just draw; 1 = draw and go interactive */

  /* open S2PLOT device */
  if (!s2open(fullscreen, stereo, argc, argv)) {
    fprintf(stderr, "Failed to open S2PLOT device - exiting!\n");
    exit(-1);
  }

  /* set the background color, and modify colour indices 0 and 1 */
  ss2sbc(0.5, 0.5, 0.5);
  ss2sfc(0.0, 0.0, 0.3);
  s2scr(0, 1., 1., 1.);
  s2scr(1, 0., 0., 0.);

  /* set world coordinate range */
  s2swin(x1, x2, y1, y2, z1, z2);

  /* draw and label a box around the points, in CYAN colour */
  s2sci(S2_PG_CYAN);
  s2sch(1.2);
  s2slw(1);
  s2box(xopt, xtick, nxsub, yopt, ytick, nysub, zopt, ztick, nzsub);

  /* plot the same parametric line using swapped axes, and colour 
   * according to a colormap */
  s2slw(3.0);
  s2icm("rainbow", 3000,3600);
  s2scir(3000, 3600);
  s2funtc(fx, fy, fz, fc, 1000, -TWOPI, TWOPI);
  s2slw(1.0);

  /* plot some points and symbols */
  x1 = XLIMIT * 0.5;
  y1 = YLIMIT * 0.5;
  z1 = ZLIMIT * 0.5;
  s2sci(S2_PG_WHITE);
  s2pt(1, &x1, &y1, &z1, 1); /* dot */

  x1 = -x1;
  s2sci(S2_PG_BLUE);
  s2pt(1, &x1, &y1, &z1, 2); /* 3d cross */

  y1 = -y1;
  s2sci(S2_PG_GREEN);
  s2pt(1, &x1, &y1, &z1, 4); /* sphere */

  z1 = -z1;
  s2sci(S2_PG_ORANGE);
  s2pt(1, &x1, &y1, &z1, 6); /* a shaded box ? */

  x1 = -x1;
  s2sci(S2_PG_YELLOW);
  s2pt(1, &x1, &y1, &z1, 0); /* a wireframe box ? */

  /* try an arrow or two */
  s2sci(S2_PG_CYAN);
  x1 = y1 = z1 = -1.0;
  x2 = y2 = z2 = 1.0;
  s2arro(x1, y1, z1, x2, y2, z2);
  s2sci(S2_PG_RED);
  z2 = 24.0;
  x1 = -3.0;
  y1 = 8.0;
  y2 = -6.7;
  s2slw(3);
  s2sch(2.4);
  s2sah(1, 60.0, 0.8);
  s2arro(x1, y1, z1, x2, y2, z2);

  /* try a few disks */
  s2sci(S2_PG_WHITE);
  s2diskxy(0, 0, 0, 1, 3);
  s2sci(S2_PG_LTGRAY);
  s2diskxz(0, 0, 0, 0.0, 0.7);
  s2sci(S2_PG_BLUE);
  s2diskyz(0, 5, 3, 2.9, 3.0);

  /* now some circles */
  s2slw(2.0);
  s2sci(S2_PG_RED);
  s2circxy(0, 0, 0, 3, 40, -1);
  s2sci(S2_PG_GREEN);
  s2circxz(5, 5, 5, 6, 62, 1.0);
  s2sci(S2_PG_YELLOW);
  s2circyz(-3, -3, 0, 1.4, 6, 2.5);

  /* now a textured sphere !!! */
  ss2ssr(32);
  ns2spheret(-5., -18., -3., 5.0, 1., 0., 0., "firetile2_pow2_rgb.tga");

  /* now an "arbitrarily oriented" disk, coloured as I like */
  ns2disk(5., 18., 6., -1., -1., -1., 1., 3., 0.8, 0.4, 0.2);
  XYZ position = {5., 18., 6.};
  XYZ normal = {-1., 1., 0.5};
  COLOUR colour = {0.4, 0.2, 0.8};
  ns2vdisk(position, normal, 0.0, 2.0, colour);
  
  /* flush graphics and enter interactive mode */
  s2show(interactive);

  /* successful run */
  return 0;

}
