/* s2uc6.c: demonstration / use case 6 for S2PLOT library.
 *
 * David Barnes, October 2005
 *
 * $Id: s2uc6.c,v 3.2 2007/04/04 22:56:39 dbarnes Exp $
 *
 * Demonstrates coloured surface plot, wedge and plot annotation,
 * text drawing functions and how to create "double-sided" labels,
 * and introduces basic dynamic geometry in the form of a bouncing
 * ball and randomly-walking dots.  Also demonstrates the PGPlot-
 * inspired string interface to choose the device via user prompt
 * and/or environment variable.
 *
 */

#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include <unistd.h>
#include "s2plot.h"

#define XLIMIT 10.0
#define YLIMIT 20.0

/* a global variable to share between the dynamic callback and the
 * numeric keypress callback */
int number;

/* here is our surface plot function */
inline float fxy(float *x, float *y) {
  return ((*x/XLIMIT) - 1.5*pow(*x/XLIMIT, 2.) + 0.3*pow(*y/YLIMIT, 3.)
	  - (*x/XLIMIT)*(*y/YLIMIT)*(*y/YLIMIT));
}

/* the callback function which will construct dynamic geometry to be 
 * drawn.
 */
void newcallback(double *time, int *keycount) {
  static int initialised = 0;
  static double phase = 0.;

  /* storage for our point population */
  static float *x = 0, *y = 0, *z = 0;
  int i;

  /* fetch bounds of our world */
  float x1, x2, y1, y2, z1, z2;
  s2qwin(&x1, &x2, &y1, &y2, &z1, &z2);

  if (!initialised) {
    x = (float *)calloc(100, sizeof(float));
    y = (float *)calloc(100, sizeof(float));
    z = (float *)calloc(100, sizeof(float));
    for (i = 0; i < 100; i++) {
      x[i] = x1 + rand() / (float)RAND_MAX * (x2 - x1);
      y[i] = y1 + rand() / (float)RAND_MAX * (y2 - y1);
      z[i] = z1 + rand() / (float)RAND_MAX * (z2 - z1);
    }
    initialised = 1;
  } else {
    for (i = 0; i < 100; i++) {
      x[i] += (rand() / (float)RAND_MAX - 0.5) * (x2 - x1) / 133.0;
      y[i] += (rand() / (float)RAND_MAX - 0.5) * (y2 - y1) / 133.0;
      z[i] += (rand() / (float)RAND_MAX - 0.5) * (z2 - z1) / 133.0;
    }
  }

  /* draw the randomly-jittering dots */
  s2sci(S2_PG_WHITE);
  s2slw(1);
  s2pt(100, x, y, z, 1);

  /* draw the ball following a lissajous-like figure in 3d space */
  /* phase is between 0 and 10 seconds */
  //double phase = *time - (int)(*time / 10.0) * 10.0;
  
  /* new version: step per call rather than per time interval */
  phase += 0.05;

  s2sci(S2_PG_CYAN);
  s2pt1((x1 + x2) * 0.5 + (x2 - x1) * 0.5 * 
	cos(phase / 10.0 * 3.14159 * 2.0),
	(y1 + y2) * 0.5 + (y2 - y1) * 0.5 * 
	sin(phase / 3.3333 * 3.14159 * 2.0), 
	(z1 + z2) * 0.5, 4);

  if (number >= 0) {
    char string[80];
    sprintf(string, "\nlast key press = %d", number);
    s2iden(string);
    if (number > 0) {
      /* sleep for (number - 1) tenths of a second */
      unsigned long sleeptime = (number - 1) * 100000;
      usleep(sleeptime);
    } else {
      /* disable callback */
      /* this beautiful thing is now possible: switch the callback off!
       * the only way for it to come back on is for the user to press the
       * animation key ("z").
       */
      cs2dcb();
    }
  }

  /* you could even change the callback function if you want!, eg.
   * cs2scb(anotherfn);
   */
}

/* a (very basic) callback function to handle numeric key presses */
void numcb(int *num) {
  number = *num;
}


/* main function definition */
int main(int argc, char **argv) {
  /* open S2PLOT device using env var or prompting */
  if (!s2opend("/?", argc, argv)) {
    fprintf(stderr, "Failed to open S2PLOT device - exiting!\n");
    exit(-1);
  }

  /* initialise the keypress indicator */
  number = -1;

  /* install a cool colormap */
  s2icm("iron", 1000, 1400);
  s2scir(1000, 1400);

  /* plot function */
  /* because ctl=1, s2funxy will auto scale and label the plot */
  s2sci(S2_PG_MAGENTA);
  s2funxy(fxy, 20, 20, -XLIMIT, XLIMIT, -YLIMIT, YLIMIT, 1);
  s2sci(S2_PG_WHITE);
  s2lab("x", "y", "f(x,y)", "s2funxy used to plot the surface z=f(x,y)");
  s2iden("\nAdditional information here, and on ...\n ... the next line!");

  // 1. find out what s2funxy did with the coordinate system
  float x1, x2, y1, y2, z1, z2;
  s2qwin(&x1, &x2, &y1, &y2, &z1, &z2);

  // big assumption here is that z1, z2 are the datamin and datamax
  float dmin = z1;
  float dmax = z2;

  s2wedge("Jy", 0.03, 0.06, 0.03, 0.41, dmin, dmax, S2_PG_RED);

  /* now try some text labels of our own */
#define XYLAB "XY Label qg YX"
#define HOVER 0.02
  s2sch(3.5);
  /* obtain padded bounding box of the proposed xy label and draw a rect */
  s2qtxtxy(&x1, &x2, &y1, &y2, 0., 0., 1.00, XYLAB, 0.2);
  s2sci(S2_PG_DKGRAY);
  s2rectxy(x1, x2, y1, y2, 1.00);
  /* change colour and write label */
  s2sci(S2_PG_YELLOW);
  s2textxy(0.0, 0.0, 1. + HOVER, XYLAB);
  /* and here's how to draw on the back of the rectangle */
  /* (use zero pad to find starting point) */
  s2qtxtxy(&x1, &x2, &y1, &y2, 0., 0., 1.00, XYLAB, 0.0);
  s2textxyf(x2, 0.0, 1. - HOVER, -1., 1., XYLAB);
  /* cannot use y1 in above as that includes offset from baseline for 
   * chars like q, g, etc. */

  /* flip the xz label for fun */
#define XZLAB "XZ label goes here!"
  s2sch(5.0);
  s2qtxtxzf(&x1, &x2, &z1, &z2, 0., 25., 0., -1., 1., XZLAB, 0.2);
  s2sci(S2_PG_DKGRAY);
  s2rectxz(x1, x2, z1, z2, 25.);
  s2sci(S2_PG_CYAN);
  s2textxzf(0.0, 25.0+HOVER*10., 0.0, -1.0, 1.0, XZLAB);
  s2qtxtxzf(&x1, &x2, &z1, &z2, 0., 25., 0., -1., 1., XZLAB, 0.0);
  s2textxzf(x2, 25.0-HOVER*10., 0.0, 1.0, 1.0, XZLAB);

  /* don't flip this one, stretch it instead! */
#define YZLAB "Put it there."
  s2sch(1.5);
  s2qtxtyzf(&y1, &y2, &z1, &z2, -13., 0., 0., 2., 0.7, YZLAB, 0.4);
  s2sci(S2_PG_LTGRAY);
  s2rectyz(y1, y2, z1, z2, -13.);
  s2sci(S2_PG_BLUE);
  s2textyzf(-13.0+HOVER*5, 0.0, 0.0, 2., 0.7, YZLAB);
  s2qtxtyzf(&y1, &y2, &z1, &z2, -13., 0., 0., 2.0, 0.7, YZLAB, 0.0);
  s2textyzf(-13.0-HOVER*5, y2, 0.0, -2., 0.7, YZLAB);

  /* install a callback function for dynamic geometry */
  cs2scb(&newcallback);

  /* install a callback function to handle numeric key presses */
  cs2sncb(&numcb);

  /* flush graphics and enter interactive mode */
  s2show(1);

  /* successful run */
  return 0;

}
