/* volrender.c: demonstration of S2PLOT volume rendering and isosurfaces
 *
 * David Barnes, February 2007
 *
 * $Id: volrender.c,v 1.2 2007/04/04 22:56:39 dbarnes Exp $
 *
 */

#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include "s2plot.h"

#define BWIDTH 40
int nx, ny, nz;
float ***grid;
float tr[12];
int vrid, isoid;

void callback(double *time, int *keycount);

int main(int argc, char **argv) {

  /* grid dimensions */
  nx = BWIDTH;
  ny = BWIDTH;
  nz = BWIDTH;

  /* transpose matrix for volume rendering and isosurface */
  /* offsets */
  tr[0] = 0.0;
  tr[4] = 0.0;
  tr[8] = 0.0;
  /* increments */
  tr[1] = 1.0;
  tr[6] = 1.0;
  tr[11] = 1.0;
  /* cross terms */
  tr[2] = tr[3] = tr[5] = tr[7] = tr[9] = tr[10] = 0.;

  /* check for valid usage */
  if (argc != 1) {
    fprintf(stderr, "usage: %s\n", argv[0]);
    exit(-1);
  }

  /* open S2PLOT device */
  if (!s2opend("/?", argc, argv)) {
    fprintf(stderr, "Failed to open S2PLOT device - exiting!\n");
    exit(-1);
  }

  /* set up coordinate system */
  s2svp(-0.7, 0.7, -0.7, 0.7, -0.7, 0.7);
  s2swin(0, nx-1, 0, ny-1, 0, nz-1);
  char *opt = "BCDETS";
  /* draw and label a box around the points, in a non-green-ish colour */
  s2sci(S2_PG_CYAN);
  s2sch(0.5);
  s2slw(1);
  s2box(opt, 0.0, 0, opt, 0.0, 0, opt, 0.0, 0);

  /* allocate and generate the data grid */
  grid = (float ***)malloc(nx * sizeof(float **));
  int i, j, k;
  float x, y, z;
  for (i = 0; i < nx; i++) {
    grid[i] = (float **)malloc(ny * sizeof(float *));
    x = (float)(i - nx/2) / (float)(nx - 1);
    for (j = 0; j < ny; j++) {
      grid[i][j] = (float *)malloc(nz * sizeof(float));
      y = (float)(j - ny/2) / (float)(ny - 1);
      for (k = 0; k < nz; k++) {
	z = (float)(k - nz/2) / (float)(nz - 1);
	grid[i][j][k] = x*x*x + y*y - z*z*z*z;
	grid[i][j][k] += 8. * exp(-16. * x*x - 18.*y*y - 20.2 *z * z);

	if ((x > -0.3) && (x < -0.0) &&
	    (y > 0.2) && (y < 0.4) &&
	    (z > -0.3) && (z < -0.2)) {
	  grid[i][j][k] += 6.0;
	}

	/* highlight the edges of the volume */
#define X_b 0
#define X_e (nx-1-X_b)
#define Y_b 0
#define Y_e (ny-1-Y_b)
#define Z_b 0 
#define Z_e (nz-1-Z_b)
	if (((i == X_b || i == X_e) && 
	     (j == Y_b || j == Y_e || k == Z_b || k == Z_e)) ||
	    ((k == Z_b || k == Z_e) && 
	     (j == Y_b || j == Y_e))) {
	  grid[i][j][k] = 12.;
	}

      }
    }
  }

  /* create the volume rendering "object" */
  vrid = ns2cvr(grid, nx, ny, nz, 
		0, nx-1, 0, ny-1, 0, nz-1,
		tr, 's', 0.6, 9., 0.0, 0.6);
  
  /* create the isosurface "object" */
  isoid = ns2cis(grid, nx, ny, nz, 0, nx-1, 0, ny-1, 0, nz-1,
		 tr, 0., 1, 't', 1.0, 0.3, 0.3, 0.3);

  cs2scb(callback);

  ss2srm(SHADE_FLAT);
  COLOUR amb = {0.8, 0.8, 0.8};
  ss2sl(amb, 0, NULL, NULL, 0);

  s2icm("rainbow", 2000, 3000);
  s2scir(2000, 3000);

  /* show the crosshair */
  ss2txh(1);

  /* show the environment */
  s2show(1);

  return (0);
}

void callback(double *time, int *keycount) {
  static int i, j, k;
  static float level;

  /* modify the data grid */
  for (i = 0; i < nx; i++) {
    for (j = 0; j < ny; j++) {
      for (k = 0; k < nz; k++) {
	grid[i][j][k] += 0.03 * sin(*time + (float)(i)/(float)(nx) * 2 * M_PI);
	grid[i][j][k] += 0.04 * cos(*time * 0.9 + (float)(j)/(float)(ny) * 
				    2 * M_PI);
	grid[i][j][k] -= 0.02 * sin(*time * 2.3 + (float)(k)/(float)(nz) * 
				    2 * M_PI);
      }
    }
  }

  /* draw the volume rendering */
  ds2dvr(vrid, 1);

  /* update the isosurface level and draw the isosurface */
  level = (*keycount % 9) + 1. - 1.0e-5;
  if (level > 1.) {
    ns2sisl(isoid, 11. - level);
    ns2dis(isoid, 1);
  }

}
